# Input

Oncodrive3D analyse patterns of somatic mutations at the cohort level, and relies on two primary input files:

---
- **Input mutations** (`required`): Mutation file that can be either:
   - **<input_maf>**: A Mutation Annotation Format (MAF) file annotated with consequences (e.g., by using [Ensembl Variant Effect Predictor (VEP)](https://www.ensembl.org/info/docs/tools/vep/index.html)).
   - **<input_vep>**: The unfiltered output of VEP including annotations for all possible transcripts.

- **<mut_profile>** (`optional`): Dictionary including the normalized frequencies of mutations (*values*) in every possible trinucleotide context (*keys*), such as 'ACA>A', 'ACC>A', and so on.

---

## Input Mutations

### MAF File

A [Mutation Annotation Format (MAF)](https://docs.gdc.cancer.gov/Data/File_Formats/MAF_Format/#introduction) file that encompasses all somatic mutations identified within a specific cohort and their annotations (e.g., annotated by using Ensembl Variant Effect Predictor (VEP)).

For example, after performing variant calling, it can be generated by VEP using a Variant Call Format (VCF) file as input:

```
vep --dir <vep_data> -i <input_vcf> --offline --cache -o <output>.vep.tsv \
    --species homo_sapiens --assembly GRCh38 --fork 8 --symbol --protein \ 
    --tab --canonical --pick
```

The output of VEP should be filtered so that each mutation is mapped to a single transcript. Additionally, it should be parsed to conform to the MAF format and include at least the following fields:

- **Hugo_Symbol**: HUGO symbol for the gene.
- **Variant_Classification**: Translational effect of variant allele.
- **Transcript_ID**: Ensembl ID of the transcript affected by the variant.
- **HGVSp_Short**: The protein sequence of the variant in HGVS recommended format using 1-letter amino-acid codes.

### VEP File

To maximize the number of matching transcripts between the input mutations and the AlphaFold predicted structures used by Oncodrive3D, it is recommended to USE the unfiltered output of VEP (generated using the VEP command described in the [MAF file section](#maf-file)) as input.
In this case:

- The file should include all possible transcripts for each mutation without filtering.
- The `--o3d_transcripts` and `--use_input_symbols` flags must be included when running the `oncodrive3d run` command to ensure proper processing and transcript matching with Oncodrive3D's pre-built datasets.

---

> [!WARNING]
> If the VEP output includes header lines starting with ## (commonly found in the VEP output format), it is recommended to remove these lines to ensure compatibility with Oncodrive3D. You can do this with the following command:
> 
> ```
> grep -v '^##' <cohort>.vep.tsv > <cohort>.vep.tsv
> ```

---

## Mutation Profile

The mutation profile of a cohort rapresents the count or the normalized frequencies of mutations in every possible k-nucleotide (e.g., trinucleotide or pentanucleotide) contexts.

The mutation profile used by Oncodrive3D is a dictionary (json file) including the frequency of mutations (*values*) of the cohort in 192 trinucleotide contexts (*keys*), normalized by the trinucleotide bias:

```json
{
    "GCA>G": 0.02424271083094251,
    "AGC>A": 0.023005887103025254,
    "ACG>T": 0.037613802858829135,
    "CGA>C": 0.10691031051670515,
    "GAC>G": 0.017846071811001615,
    "TTC>A": 0.024003748061871697,
    "CTT>G": 0.024149863672267024,
    "GGA>T": 0.011178562948734577,
    "AGG>C": 0.010654720767868876,
    "GGG>C": 0.012031686292218055,
    "CAA>T": 0.014478959792844522,
    "TGA>A": 0.01255651801972085,
    "GGA>A": 0.011178562948734577,
    "CGA>A": 0.03563677017223505,
    "TCC>T": 0.011158347971568658,
    "GCC>A": 0.010952316565906438,
    // ...
}
```

The mutation profile can be computed using BGSignature (detailed below) or other bioinformatics softwares.  

### Create the Mutation Profile with BGSignature

1. Install BGSignature:
    ```
    pip install bgsignature
    ```

2. Get the count of trinucleotides:
    ```
    bgsignature count -r <regions_file> -s 3 -g GRCh38 --cores 8 --collapse \
                    --exclude-N -o count.gz
    ```

3. Get the frequency of mutations normalized by trinucleotide bias:
    ```
    bgsignature normalize -m <mutations_file> -r <regions_file> --normalize count.gz \ 
                        -s 3 -g GRCh38 --collapse --cores 8 -o <cohort>.sig.json
    ```

To compute the mutation profile with BGSignature two main files are required:

- **mutations_file**: Tab Separated File (TSV) including an header and at least the following columns: 
    - `CHROMOSOME`
    - `POSITION`
    - `REF`
    - `ALT`
- **regions_file**: Tab Separated File (TSV) including the coordinates of the genomic regions (e.g., a panel of genes, the whole exome, or the whole genome) that the user wants to consider for computing the mutation profile. This file should include an header and at least the following columns: 
    - `CHROMOSOME`
    - `START`
    - `END`
    - `ELEMENT`  
    
#### Create the Regions File  

The regions file can be generated using BGReference. Below is an example of a Python script to create a regions file for a specified genome and k-mer size.

1. Install BGReference:
    ```
    pip install bgreference
    ```

2. Save the following Python script as get_regions_file.py:

    ```python
    import sys
    from bgreference import refseq

    CHR = [str(i) for i in range(1, 20)] + ['X', 'Y', 'M']


    def compute_sizes(genome, kmer):
        sizes = []
        for chr_ in CHR:
            seq = refseq(genome, f"chr{chr_}", start=(1 + kmer // 2), size=None)
            sizes.append(tuple(map(str, (chr_, 1 + kmer // 2, len(seq) - kmer // 2))))
        return sizes


    def write(sizes):
        print('\t'.join(('CHROMOSOME', 'START', 'END')))
        for s in sizes:
            print('\t'.join(s))

    if __name__ == '__main__':
        genome = sys.argv[1]
        kmer = int(sys.argv[2])
        write(compute_sizes(genome, kmer))
    ```

3. Run the script:

    ```
    python3 get_regions_file.py hg38 3 > hg38_wg_regions.tsv
    ```

# Ouput

The `oncodrive3d run command` performs the 3D clustering analysis, generating both main and supplementary output files. The main output files include a gene-level output and a residue-level output, both summarizing the results of the analysis. The supplementary output files include the processed input files and a processed file derived from the Oncodrive3D built datasets, containing information on the genes being analyzed.

## Main Output

### Gene-level output

CSV file (`<cohort>.3d_clustering_genes.csv`) containing the results of the analysis at the gene level. The genes (rows) are sorted by ascending order based on significance and deviation from neutrality. 

It includes the following fields:

- **Gene**: HUGO symbol or identifier of the gene being analyzed.
- **Uniprot_ID**: Identifier for the gene's protein product in the UniProt database.
- **pval**:  The p-value indicating the statistical significance of the gene in the 3D clustering analysis (lowest p-value among the residues of the gene).
- **qval**: Adjusted p-value (q-value) to control for false discovery rate (FDR) using the Benjamini-Hochberg method.
- **C_gene**: Binary label indicating if the gene is detected to be significant (by default if `q-value < 0.01`).
- **C_pos**: List of protein positions of the gene clusters.
- **C_label**: List of labels indicating the clump to which each cluster is grouped.
- **Pos_top_vol**: Position of the most significant cluster.
- **Score_obs_sim_top_vol**: 3D clustering score of the gene (normalized 3D clustering score of the residue with the lowest p-value; if multiple residues have the same lowest p-value, the maximum normalized 3D clustering score among those residues is used).
- **Mut_in_gene**: Number of missense mutations in the gene.
- **Clust_mut**: Number of missense mutations in significant clusters.
- **Clust_res**: Number of residues detected as significant clusters.
- **Mut_in_top_vol**: Number of missense mutations in the most significant cluster (in the volume of the residue with the lowest p-value).
- **Mut_in_top_cl_vol**: Number of missense mutations in the clusters of the most significant clump (in the volume of residues of the clump that includes the residue with the lowest p-value).
- **PAE_top_vol**: Weighted average predicted aligned error (PAE) between the residues in the volume of the most significant cluster.
- **pLDDT_top_vol**: Weighted average predicted local distance difference test (pLDDT) between the residues in the volume of the most significant cluster.
- **pLDDT_top_cl_vol**: Weighted average pLDDT between the residues in the volume of the most significant clump.
- **Ratio_not_in_structure**: Diagnostic field rapresenting the proportion of mutations in the input file that are mapped to positions not covered by the structure.
- **Ratio_WT_mismatch**: Diagnostic field rapresenting the proportion of mutations in the input file where the wild-type (WT) amino acid does not match the corresponding residue in the structural model.
- **Mut_zero_mut_prob**: Diagnostic field rapresenting the proportion of mutations in the input file that are assigned a mutation probability of zero. This field is relevant only when a mutability configuration file is provided as input (`-m`, `--mutability_config_path`).  
- **Pos_zero_mut_prob**: Diagnostic field reporting the mutated protein position with assigned a mutation probability of zero. This field is relevant only when a mutability configuration file is provided as input (`-m`, `--mutability_config_path`).  
- **Cancer**: Cancer type.
- **Cohort**: Cohort name.
- **F**: Fragment of the AlphaFold predicted structure (1 = no fragmentation, *n*M = merged from *n* fragments).
- **Transcript_ID**: Ensembl transcript ID provided in the input file.
- **O3D_transcript_ID**: Ensembl trasncript ID mapped to the given gene in the Oncodrive3D built datasets.
- **Transcript_status**: Transcripts mapping status indicating whether the transcript IDs matches between the input data and the Oncodrive3D built datasets. Possible values are:
    - `Input_missing`: Transcript ID is missing from the input file.
    - `O3D_missing`: Transcript ID is missing from the Oncodrive3D built datasets.
    - `Mismatch`: Transcript ID in the input file does not match the transcript ID in the Oncodrive3D built datasets.
    - `Match`: Transcript ID in the input file matches the transcript ID in the Oncodrive3D built datasets.
- **Status**: Processing status for 3D clustering analysis. Possible values are:
    - `Processed`: The gene has been successfully processed, with a 3D clustering score and a p-value assigned.
    - `No_mut`: The gene contains one or no mutations.
    - `No_density`: The maximum number of mutations in the spatial volume of any residue is one or less.
    - `No_ID_mapping`: No corresponding UniProt ID is found for the given gene in the Oncodrive3D built datasets.
    - `Cmap_not_found`: The contact map for the gene's structural data is not available.
    - `Mut_not_in_structure`: The proportion of mutations mapped to positions outside the boundaries of the provided PDB structure exceeds the threshold for mapping issues
    (`--thr_mapping_issue`, default: `0.1`).
    - `WT_mismatch`: The proportion of mutations in the input file where the wild-type amino acid does not match the corresponding residue in the structural model exceeds the threshold for mapping issues (`--thr_mapping_issue`, default: `0.1`).
    - `Mut_with_zero_prob`: The proportion of mutations mapped to positions with a mutation probability of zero exceeds the threshold for mapping issues (`--thr_mapping_issue`, default: `0.1`).
    - `No_mutability`: A mutability configuration file (`-m`, `--mutability_config_path`) was provided, but accurate information on the DNA sequence of the gene is missing in the Oncodrive3D built datasets.
    - `NA_miss_prob`: The missense probability vector computed for the protein includes NAs values.
    - `Fragmented`: The protein structure is predicted as fragments by AlphaFold, and the analysis of fragmented structures is not enabled (`-f`, `--no_fragments`).


### Residue-level output
  
CSV file (`<cohort>.3d_clustering_pos.csv`) containing the results of the analysis at the level of mutated residues. Each row corresponds to a mutated position within a gene and includes detailed information for each potential mutational cluster.

It includes the following fields:

- **Gene**: HUGO symbol or identifier of the gene being analyzed.
- **Uniprot_ID**: Identifier for the gene's protein product in the UniProt database.
- **Pos**: Protein position.
- **Mut_in_gene**: Number of missense mutations in the gene.
- **Mut_in_res**: Number of missense mutations in the residue.
- **Mut_in_vol**: Number of missense mutations in the volume of the residue.
- **Score**: 3D clustering score for the residue.
- **Score_obs_sim**: Normalized 3D clustering score for the residue.
- **pval**: The p-value of the residue in the 3D clustering analysis.
- **C**: Binary label indicating whether the cluster at that residue is significant (1) or not (0). A cluster is marked as significant either because it meets the significance criteria directly or because it has been rescued by contributing mutations to another significant cluster.
- **C_ext**: Binary label indicating whether the cluster has been rescued by contributing mutations to another significant cluster (1) or if it was significant on its own (0).
- **Clump**: Identifier for the clump to which the cluster at that residue has been assigned.
- **Rank**: Rank used to perform the calculation of the normalized 3D clustering score and p-values. 
- **Mut_in_cl_vol**: Number of missense mutations in the clusters of the clump to which the cluster has been assigned.
- **Res_in_cl**: Positions of the clusters of the clump to which the cluster has been assigned.
- **PAE_vol**: Weighted average predicted aligned error (PAE) of the residues in the volume of the cluster.
- **pLDDT_res**: Predicted local distance difference test (pLDDT) of the residue.
- **pLDDT_vol**: Weighted average pLDDT between the residues in the volume.
- **pLDDT_cl_vol**: pLDDT between the residues in the volume of the most significant cluster.
- **Cancer**: Cancer type.
- **Cohort**: Cohort name.


## Supplementary Output